#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>         /* for read, write, close */
#include <sys/ioctl.h>      /* for ioctl */
#include <sys/types.h>      /* for open */
#include <sys/stat.h>       /* for open */
#include <fcntl.h>          /* for open */

#include <signal.h>

#include "driver/urc.h"          /* Linux specific part */
#include "driver/urcc.h"          /* Linux specific part */

#define numOfControllers 2
static char *devfiles[] = {"/dev/urc0", "/dev/urc1"};

void exit_program();

int quit_flag = 1;
int fd, fds[numOfControllers];

int main(int argc, char **argv){
  struct urcout cmd[numOfControllers];
  struct urcin buf, data[numOfControllers];
  short n, i, j;
  unsigned short duty[2];

  signal(SIGINT, exit_program);

  /* $B%G%P%$%9$N%*!<%W%s(B */
  for (i=0; i<numOfControllers; i++) {
    if ((fd = open(devfiles[i], O_RDWR)) == -1) {
      fprintf(stderr, "%s: Open error\n", devfiles[i]);
      exit(1);
    }
    if (ioctl(fd, URC_READ) < 0){
      fprintf(stderr, "ioctl: URC_READ error\n");
      exit(1);
    }
    if (read(fd, &buf, sizeof(buf)) != sizeof(buf)) {
      fprintf(stderr, "Read size mismatch.\n");
      exit(1);
    }

    if (buf.magicno == 0) {
      fds[0] = fd;
      fprintf(stderr, "Found controller #0.\n");
    } else if (buf.magicno == 1) {
      fds[1] = fd;
      fprintf(stderr, "Found controller #1.\n");
    } else {
      fprintf(stderr, "Wrong magic no: %d.\n", buf.magicno);
      exit(1);
    }
  }
  j=0;

  while(quit_flag) {
/* $B%j%H%k%(%s%G%#%"%s$+%S%C%0%(%s%G%#%"%s$+$rD4$Y%G!<%?$NG[CV$rJQ99(B */
#if __BYTE_ORDER == __LITTLE_ENDIAN
  duty[0] = 0x05D8;
  duty[1] = 0x1387;
#else
  duty[0] = 0xD805;
  duty[1] = 0x8713;
#endif
       if(j>500) j=0;
       for (n=0; n<numOfControllers; n++) {
         cmd[n].duty[0] = cmd[n].duty[1] = cmd[n].duty[2] = cmd[n].duty[3] = duty[n];

         /* $B%i%8%3%s%5!<%\$NL\I8CM$NJQ99(B */
         if (ioctl(fds[n], URC_SET) < 0){
  	 fprintf(stderr, "ioctl: URC_SET error\n");
  	 exit(1);
         }

         if (write(fds[n], &cmd[n], sizeof(cmd[n])) < 0) {
  	 fprintf(stderr, "write error\n");
  	 exit(1);
         }
     
         /* 20ms$B$N(Bwait */
         usleep(20000);

         /* A/D$B%3%s%P!<%?$NJQ49CM$H%^%8%C%/(BNo.$B$N<hF@(B */
         if (ioctl(fds[n], URC_READ) < 0){
  	 fprintf(stderr, "ioctl: URC_READ error\n");
  	 exit(1);
         }

         if ((i = read(fds[n], &data[n], sizeof(data[n]))) != 64) {
  	 fprintf(stderr, "Warning: read size mismatch (%d!=%d).\n", i, sizeof(data[n]));
  	 continue;
         }
       
         printf("DEV_%d:\r\n", n);
         printf("CH0:0x%04X\tCH1:0x%04X\tCH2:0x%04X\tCH3:0x%04X\tMAGIC_NO:0x%04X",data[n].ad[0], data[n].ad[1], data[n].ad[2], data[n].ad[3], data[n].magicno);
         printf("\n");
     
       /* 20ms$B$N(Bwait */
       usleep(20000);
     }
   }
   
   close(fds[0]);
   close(fds[1]);

  return 0;
}

void exit_program(sig, code, scp, addr)
int sig;
int code;
struct sigcontext *scp;
char *addr;
{
  quit_flag = 0;
  fprintf(stderr, "kill signal is received\n");
}
